; Pensate V1.0
; By Robin Kay
;

; You may not distribute a modified version of this program. However, you
; may take code from or base your program on this code providing that the
; resulting program is at least 50% of your own work.

.nolist
#include "tse.inc"
.list
.org    usermem-3
.db     $BB,$6D
 ret

.db "TSE",1              ; header
prgtitle:
.db " Pensate", 0        ; title
.dw 120                  ; stack required

init:

 ; Init objects
 ld a, 5
 ld (objects),a

start:
 ; Dispaly main menu
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl
 ld hl, prgtitle + 1
 call putStr
 ld hl, author
 call putStr
 bcall(_newline)
 ld hl, txt1
 call putStr
 ld hl, txt2
 call putStr
 bcall(_newline)
 bcall(_newline)
 ld hl, objtxt
 bcall(_puts)
 ld hl, (objects)
 ld h, 0
 bcall(_dispHL)

 ; Poll for keypad on menu
loop:
 ei
 halt ; Save power
 bcall(_getCSC)

 ; Process keys
 ld hl, start
 push hl
 cp sk2nd
 jr z, game
 cp skAdd
 jr z, incobj
 cp skSub
 jr z, decobj
 pop hl

 ; Yield
 cp skStat
 jr nz, dontyield
 call _tseforceyield
 jr start
dontyield:
 jr loop

; Increase number of objects
incobj:
 ld a, (objects)
 inc a
 cp 21
 ret z
 ld (objects), a
 ret

; Decrease number of objects
decobj:
 ld a, (objects)
 dec a
 cp 0
 ret z
 ld (objects), a
 ret

; Main game
game:

 ; Init board
 ld hl, board
 ld b, 64
 xor a
initboard:
 ld (hl), a
 inc hl
 djnz initboard

 ; Populate board
 ld a, (objects)
 ld b, a
popboard:
 push bc
already_there:

 ; Find location
 ld b, 64-8 ; Don't populate bottom row!
 call ionRandom
 ld e, a
 ld d, 0
 ld hl, board
 add hl, de

 ; Check if in use
 ld a, (hl)
 cp 0
 jr nz, already_there

 ; Place counter
 ld b, 10
 call ionRandom
 inc a
 ld (hl), a
 pop bc
 djnz popboard

 ; Place user counter
 ld b, 8
 call ionRandom
 ld l, a
 ld h, 0
 ld de, board+(64-8)
 add hl, de
 ld a, 0Bh
 ld (hl), a

gameloop:

 ; Clear graph buffer
 bcall(_GrBufClr)

 ; Reset first byte of userloc
 xor a
 ld (userloc), a

 ; Draw the board
 ld hl, board + 63
 ld b, 8
drawLoop:
 push bc
 ld c, b
 ld b, 8
innerDrawLoop:

 ; Get type of counter at square
 push hl
 ld a, (hl)
 cp 0Bh
 jr nz, notuser
 ld (userloc), bc
notuser:
 sla a
 sla a
 sla a
 ld e, a
 ld d, 0
 ld ix, counters
 add ix, de

 ; Draw counter
 push bc
 dec b
 sla b
 sla b
 sla b
 dec c
 sla c
 sla c
 sla c
 ld a, b
 ld l, c
 ld b, 8
 call ionPutSprite
 pop bc
 pop hl

 ; Next square
 dec hl

 ; Loop
 djnz innerDrawLoop
 pop bc
 djnz drawLoop

 ; Display on screen
 call ionFastCopy

 ; Draw line
 ld bc, (65*256)+0
 ld de, (65*256)+62
 ld h, 1
 bcall(_iline)

 ; Check if lost
 ld a, (userloc)
 cp 0
 jr nz, notlost
 ld hl, lost
 jr pause
notlost:

 ; Check for win
 ld a, (userloc)
 cp 1
 jr nz, notwon
 ld hl, win
 jr pause
notwon:

 ; Get key press
keyloop:
 ei
 halt ; Save power
 bcall(_getCSC)

 ; Quit? Switch?
 cp skMode
 ret z
 cp skStat
 jr nz, dontyield2
 call _tseForceYield
 jp gameloop
dontyield2:

 ; Move?
 ld hl, 0
 cp skUp
 jr z, up
 cp skDown
 jr z, down
 cp skLeft
 jr z, left
 cp skRight
 jr z, right

 ; Loop
 jr keyloop

; Pause
pause:
 push hl
 ld hl, (6*256)+7
 ld (currow), hl
 pop hl
 set textInverse, (iy+textflags)
 bcall(_puts)
 res textInverse, (iy+textflags)
ploop:
 ei
 halt
 bcall(_GetCSC)
 cp skMode
 ret z
 cp skStat
 jr nz, ploop
 call _tseForceYield
 ret

; Process movements
right:
 inc hl
left:
 inc hl
down:
 inc hl
up:

 ; Check if user counter is wrapping (L&R)
 ld d, 255
 ld bc, (userloc)
 ld a, b
 cp 1
 jr nz, tleft
 ld d, 2
tleft:
 cp 8
 jr nz, tright
 ld d, 3
tright:
 ld a, l
 cp d
 jr z, keyloop

 ; Check if user counter is wrapping (U&D)
 ld d, 255
 ld a, c
 cp 1
 jr nz, tup
 ld d, 0
tup:
 cp 8
 jr nz, tdown
 ld d, 1
tdown:
 ld a, l
 cp d
 jr z, keyloop

 ; Set offset
 ld bc, behave
 add hl, bc
 ld (offset), hl

 ; Copy to temp board
 ld hl, board
 ld de, saferam5
 ld bc, 64
 ldir

 ; Update positions
 ld b, 64
 ld hl, -1
updateLoop:
 inc hl
 push hl
 push bc

 ; Skip if no counter there
 push hl
 ld bc, board
 add hl, bc
 ld a, (hl)
 pop hl
 cp 0
 jr z, skip

 ; Push location of counter on temp board onto stack
 ex de, hl
 ld h, d
 ld l, e
 ld bc, saferam5
 add hl, bc
 ld (hl), 0
 push hl
 ex de, hl

 ; Push offset of counter on the board onto stack twice
 push hl
 push hl

 ; Get address of behavour code
 dec a
 sla a
 sla a
 ld c, a
 ld b, 0
 ld hl, (offset)
 add hl, bc

 ; Get offset of counter's new location on the board
 ld a, (hl)
 pop hl
 ld c, a
 ld b, 0
 bit 7, a
 jr z, pos
 ld b, 0ffh
pos:
 add hl, bc

 ; Fix offset off counter if off board
 ld a, l
 and %00111111
 ld l, a
 ld h, 0

 ; Get address of counter's new location on temp board
 ld bc, saferam5
 add hl, bc

 ; Get address of counter's old location on the board
 pop de
 ld bc, board
 ex de, hl
 add hl, bc
 ex de, hl

 ; Check if user counter is to be overwritten
 ld a, (hl)
 cp 0Bh
 jr nz, notuser2
 ld (hl), 0
 push hl
 ld bc, (-saferam5)+board
 add hl, bc
 ld (hl), 0
 pop hl
notuser2:

 ; Check if location is free and move if it is
 cp 0
 ld a, (de)
 pop de
 jr z, move
 ex de, hl
move:
 ld (hl), a
skip:

 ; Loop
 pop bc
 pop hl
 djnz updateLoop

 ; Copy temp board onto real board
 ld hl, saferam5
 ld de, board
 ld bc, 64
 ldir

 ; Finish
 jp gameloop

; Display string and then advance a line onwards
putStr:
 bcall(_puts)
 bcall(_newline)
 ret

behave:
; Left counter (0)
.db -1+ 0 ;U
.db -1+ 0 ;D
.db -1+ 0 ;L
.db -1+ 0 ;R
; Right counter (1)
.db  1+ 0 ;U
.db  1+ 0 ;D
.db  1+ 0 ;L
.db  1+ 0 ;R
; Up counter (2)
.db  0+-8 ;U
.db  0+-8 ;D
.db  0+-8 ;L
.db  0+-8 ;R
; Down counter (3)
.db  0+ 8 ;U
.db  0+ 8 ;D
.db  0+ 8 ;L
.db  0+ 8 ;R
; Axis Inverted counter (4)
.db -1+ 0 ;U
.db  1+ 0 ;D
.db  0+-8 ;L
.db  0+ 8 ;R
; Linier Direction Inverted counter (5)
.db  0+ 8 ;U
.db  0+-8 ;D
.db  1+ 0 ;L
.db -1+ 0 ;R
; Vertically Inverted Diagonal (6)
.db -1+ 8 ;U
.db  1+-8 ;D
.db  1+ 8 ;L
.db -1+-8 ;R
; Horizontally Inverted Diagonal (7)
.db  1+-8 ;U
.db -1+ 8 ;D
.db -1+-8 ;L
.db  1+ 8 ;R
; Inverted Knight (8)
.db -1+ 16 ;U
.db  1+-16 ;D
.db -2+ 8 ;L
.db  2+-8 ;R
; Non-Inverted Knight (9)
.db -1+-16 ;U
.db  1+ 16 ;D
.db -2+-8 ;L
.db  2+ 8 ;R
; User Counter (A)
.db  0+-8 ;U
.db  0+ 8 ;D
.db -1+ 0 ;L
.db  1+ 0 ;R

counters:
; Clear (0)
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
; Left counter (1)
.db %00000000
.db %00010000
.db %00110000
.db %01111110
.db %01111110
.db %00110000
.db %00010000
.db %00000000
; Right counter (2)
.db %00000000
.db %00001000
.db %00001100
.db %01111110
.db %01111110
.db %00001100
.db %00001000
.db %00000000
; Up counter (3)
.db %00000000
.db %00011000
.db %00111100
.db %01111110
.db %00011000
.db %00011000
.db %00011000
.db %00000000
; Down counter (4)
.db %00000000
.db %00011000
.db %00011000
.db %00011000
.db %01111110
.db %00111100
.db %00011000
.db %00000000
; Axis Inverted counter (5)
.db %00001000
.db %00011100
.db %01001000
.db %11111010
.db %01011111
.db %00010010
.db %00111000
.db %00010000
; Linier Direction Inverted counter (6)
.db %00000000
.db %01001110
.db %11100010
.db %01000010
.db %01000010
.db %01000111
.db %01110010
.db %00000000
; Vertically Inverted diagonal (7)
.db %10100101
.db %01100110
.db %11100111
.db %00000000
.db %00000000
.db %11100111
.db %01100110
.db %10100101
; Horizontally Inverted diagonal (8)
.db %11100111
.db %11000011
.db %10100101
.db %00000000
.db %0000000
.db %10100101
.db %11000011
.db %11100111
; Inverted knight (9)
.db %00000000
.db %00011100
.db %00011110
.db %00011100
.db %00011000
.db %00011000
.db %00011000
.db %00000000
; Non-Inverted knight (A)
.db %00000000
.db %00111000
.db %01111000
.db %00111000
.db %00011000
.db %00011000
.db %00011000
.db %00000000
; User counter (B)
.db %00000000 
.db %00011000
.db %00111100
.db %01111110
.db %01111110
.db %00111100
.db %00011000
.db %00000000

author:
.db "By Robin Kay",0
txt1:
.db "2nd: Start Game",0
txt2:
.db "+/-: Objects",0
objtxt:
.db "Objects:",0
lost:
.db "You LOSE!",0
win:
.db "You WIN!",0

extdata
board        .equ extdata
objects      .equ board+64
offset       .equ objects+1
userloc      .equ offset+2

.end
